/*
 * url_translate.c --- do URL transmogrifications using regular
 * expresson matching and replacing
 *
 * Copyright 1994 by Theodore Ts'o <tytso@athena.mit.edu>
 *
 * This file may be distributed under the terms of the GNU Public License.
 *
 * This file was butchered by john but hopefully its still useful.  Mostly
 * changed to match the usual chimera style of memory allocation and
 * coding.  Butchered even more.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <pwd.h>

#ifdef USE_GNURX
#include <rxposix.h>
#else
#include <regex.h>
#endif

#include "url.h"

struct translate_pat
{
  char	*match;
  char	*replace;
};

#define SOME_SPACE 1024

int
URLReadTranslations(mp, list, filename)
MemPool mp;
GList list;
char *filename;
{
  FILE *f;
  char buf[SOME_SPACE];
  char m[SOME_SPACE];
  char r[SOME_SPACE];
  char t[SOME_SPACE];
  struct translate_pat *pat;
  int count = 0;
  int line = 1;

  if ((filename = FixPath(mp, filename)) == NULL) return(-1);
  if ((f = fopen(filename, "r")) == NULL) return(-1);
  
  while (!feof(f))
  {
    if (!fgets(buf, sizeof(buf), f)) break;
    if (buf[0] == '#') continue;
    if (sscanf (buf, "%[^\t]%[\t]%[^\n]", m, t, r) == 3)
    {
      pat = (struct translate_pat *)MPGet(mp, sizeof(struct translate_pat));
      pat->match = MPStrDup(mp, m);
      pat->replace = MPStrDup(mp, r);
      GListAddTail(list, pat);
      count++;
    }
    else fprintf (stderr, "Line %d invalid in %s\n", line, filename);
  }

  fclose(f);

  return(count);
}

char *
URLTranslate(mp, list, url)
MemPool mp;
GList list;
char *url;
{
  struct translate_pat *pat;
  regex_t re;
  regmatch_t *pmatch;
  MemPool tmp;
  int i;
  int match;
  char *p, *q;
  size_t size;
  char output[SOME_SPACE];

  tmp = MPCreate();
  pmatch = MPGet(tmp, sizeof(regmatch_t) * 10);

  for (pat = (struct translate_pat *)GListGetHead(list); pat != NULL;
       pat = (struct translate_pat *)GListGetNext(list))
  {
    if (regcomp(&re, pat->match, REG_EXTENDED) != 0) continue;
    
    if (regexec(&re, url, 10, pmatch, 0) == 0)
    {
      size = SOME_SPACE - 1;
      for (p = pat->replace, q = output; *p != '\0'; p++)
      {
	if (*p == '\\' && *(p+1) >= '0' && *(p+1) <= '9')
	{
	  p++;
	  match = *p - '0';
	  for (i = pmatch[match].rm_so; i < pmatch[match].rm_eo; i++)
	  {
	    *q++ = url[i];
	    if (!--size) goto finish;
	  }
	}
	else
	{
	  *q++ = *p;
	  if (!--size) goto finish;
	} 
      }

finish:
      MPDestroy(tmp);
      *q = '\0';
      regfree(&re);
      return(MPStrDup(mp, output));
    }

    regfree(&re);
  }

  MPDestroy(tmp);
  return(NULL);
}
