/*
 * load.c
 *
 * libhtml - HTML->X renderer 
 *
 * Copyright (c) 1996-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>

#include "html.h"

int
HTMLLoadURL(li, frame, url, button)
HTMLInfo li;
char *frame;
char *url;
int button;
{
  WWWRenderActionInfo ai;

  if (url == NULL) return(-1);

  if (url[0] == '#')
  {
    HTMLFindName(li, url + 1);
    return(0);
  }
  
  ai.wr = WWWBuildRequest(li->wc, url, li->burl);
  if (button == 1) ai.type = "display";
  else if (button == 2) ai.type = "download";
  
  WWWDoRenderAction(li->wn, &ai);

  return(0);
}

int
HTMLLoadAnchor(li, anchor, x, y, button, ismap)
HTMLInfo li;
HTMLAnchor anchor;
int x, y;
int button;
bool ismap;
{
  char *aurl, *url;
  MemPool mp;
  WWWRenderActionInfo ai;

  if ((url = MLFindAttribute(anchor->p, "href")) == NULL) return(-1);
  
  if (url[0] == '#')
  {
    HTMLFindName(li, url + 1);
    return(0);
  }

  if (ismap)
  {
    mp = MPCreate();
    aurl = (char *)MPGet(mp, strlen(url) + 200);
    snprintf (aurl, strlen(url) + 200, "%s?%d,%d", url, x, y);
    url = aurl;
  }
  else mp = NULL;

  ai.wr = WWWBuildRequest(li->wc, url, li->burl);
  if (mp != NULL) MPDestroy(mp);

  if (button == 1) ai.type = "display";
  else if (button == 2) ai.type = "download";
  
  WWWDoRenderAction(li->wn, &ai);

  return(0);
}

void
HTMLPrintURL(li, url)
HTMLInfo li;
char *url;
{
  WWWRequest *wr;
  size_t len;
  char buffer[BUFSIZ + 100];
  const char *msg = "Cannot handle "; /* this is the wrong thing to do! */

  if (url == NULL) return;

  if ((wr = WWWBuildRequest(li->wc, url, li->burl)) != NULL)
  {
    url = wr->url;
  }
  else
  {
    strcpy(buffer, msg);
    len = sizeof(buffer) - strlen(msg) - 1;
    strncat(buffer, url, len);
    buffer[sizeof(buffer) - 1] = '\0';
    url = buffer;
  }

  WWWSendRenderMessage(li->wn, url);

  if (wr != NULL) WWWDestroyRequest(wr);

  return;
}

void
HTMLPrintAnchor(li, anchor, x, y, ismap)
HTMLInfo li;
HTMLAnchor anchor;
int x, y;
bool ismap;
{ 
  char *url, *aurl;
  MemPool mp;

  if ((url = MLFindAttribute(anchor->p, "href")) == NULL) return;

  if (ismap)
  {
    mp = MPCreate();
    aurl = (char *)MPGet(mp, strlen(url) + 200);
    snprintf (aurl, strlen(url) + 200, "%s?%d,%d", url, x, y);
    url = aurl;
    HTMLPrintURL(li, url);
    MPDestroy(mp);
  }
  else HTMLPrintURL(li, url);


  return;
}

/*
 * HTMLAddAnchor
 */
void
HTMLAddAnchor(li, box, name, p)
HTMLInfo li;
HTMLBox box;
char *name;
MLElement p;
{
  HTMLAnchor a;

  a = (HTMLAnchor)MPCGet(li->mp, sizeof(struct HTMLAnchorP));
  a->name = name;
  a->p = p;
  a->box = box;
  GListAddHead(li->alist, a);

  return;
}

/*
 * HTMLFindName
 */
void
HTMLFindName(li, name)
HTMLInfo li;
char *name;
{
  HTMLAnchor a;
  GList list;

  list = li->alist;
  for (a = (HTMLAnchor)GListGetHead(list); a != NULL;
       a = (HTMLAnchor)GListGetNext(list))
  {
    if (a->name != NULL && strcasecmp(name, a->name) == 0)
    {
      WWWSetPosition(li->wd, a->box->x, a->box->y, true);
      return;
    }
  }

  return;
}
