/*
 * resources.h
 *
 * Copyright (C) 1993-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

typedef struct BookmarkContextP *BookmarkContext;
typedef struct AppResourcesP AppResources;
typedef struct CHeadP CHead;

typedef enum
{
  StackNew,
  StackBack,
  StackReload,
  StackReplace,
  StackRedraw,
  StackHome
} StackAction;

struct CHeadP
{
  MemPool mp;

  Widget toplevel;
  Widget file;
  Widget www;
  Widget open;
  Widget back;
  Widget reload;
  Widget cancel;
  Widget quit;
  Widget home;
  Widget addmark;
  Widget viewmark;
  Widget message;
  Widget dup;
  Widget source;
  Widget save;
  Widget openpop;
  Widget savepop;
  Widget bookpop;
  Widget url;
  Widget help;
  Widget bookmark;
  Widget search;

  GList              urls;
  GList              ourls;
  WWWDisplay         wd;
  StackAction        action;
  WWWRender          wn;
  WWWData            wp;
  WWWData            pending;
  bool               useplain;
  MemPool            tmp;
  WWWContext         wc;

  char *button1Box;           /* list of widgets in the first button box */
  char *button2Box;           /* list of widgets in the second button box */
};

struct AppResourcesP
{
  WWWContext      wc;
  Display         *dpy;
  XtAppContext    appcon;
  GList           heads;           /* list of heads */
  GList           downloads;       /* list of downloads */
  GList           views;           /* list of external views */
  CHead           *bmhead;         /* head associated with bookmarks */
  WWWRenderHooks  *plain;          /* plain text renderer */
  BookmarkContext bc;              /* bookmark routines context */
  FILE            *logfp;          /* log file */
};

/*
 * bookmark.c
 */
BookmarkContext BookmarkCreateContext _ArgProto((void));
void BookmarkDestroyContext _ArgProto((BookmarkContext));
void BookmarkAdd _ArgProto((BookmarkContext, char *, char *));
void BookmarkShow _ArgProto((BookmarkContext));

/*
 * widget.c
 */
CHead *CreateHead _ArgProto((WWWContext, char *));
void DestroyHead _ArgProto((CHead *));
Widget CreateDialog _ArgProto((Widget, char *,
			       void (*)(), void (*)(), void (*)(), XtPointer));
Widget GetDialogWidget _ArgProto((Widget));

/*
 * main.c
 */
void Quit _ArgProto((Widget, XtPointer, XtPointer));
void Open _ArgProto((Widget, XtPointer, XtPointer));
void Back _ArgProto((Widget, XtPointer, XtPointer));
void Reload _ArgProto((Widget, XtPointer, XtPointer));
void Cancel _ArgProto((Widget, XtPointer, XtPointer));
void Home _ArgProto((Widget, XtPointer, XtPointer));
void Dup _ArgProto((Widget, XtPointer, XtPointer));
void Help _ArgProto((Widget, XtPointer, XtPointer));
void AddMark _ArgProto((Widget, XtPointer, XtPointer));
void ViewMark _ArgProto((Widget, XtPointer, XtPointer));
void Bookmark _ArgProto((Widget, XtPointer, XtPointer));
void Source _ArgProto((Widget, XtPointer, XtPointer));
void Save _ArgProto((Widget, XtPointer, XtPointer));
void Search _ArgProto((Widget, XtPointer, XtPointer));
void CheckForDone _ArgProto((void));

/*
 * callback.c
 */
bool MessageCallback _ArgProto((void *, char *));
bool ActionCallback _ArgProto((void *, WWWRenderActionInfo *));
bool RedrawCallback _ArgProto((void *));
bool ResizeCallback _ArgProto((void *));

/*
 * builtin.c
 */
void InitChimeraBuiltins _ArgProto((WWWContext));

/*
 * view.c
 */
void ViewOpen _ArgProto((WWWRequest *));

/*
 * download.c
 */
void DownloadOpen _ArgProto((WWWRequest *));

/*
 * stack.c
 */
char *HeadGetURL _ArgProto((CHead *));

void HeadBack _ArgProto((CHead *));
void HeadHome _ArgProto((CHead *));
void HeadOpen _ArgProto((CHead *, WWWRequest *));
void HeadReload _ArgProto((CHead *));
void HeadCancel _ArgProto((CHead *));
void HeadRedraw _ArgProto((CHead *));
