/*
 * widget.c
 *
 * Copyright (C) 1993-1997, John D. Kilburg <john@cs.unlv.edu>
 *
 * The button table code written by Jim.Rees@umich.edu.
 * The messed up parts were written by john.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Label.h>

#include "TextField.h"

#include "common.h"

#include "MyDialog.h"

#include "WWW.h"
#include "resources.h"

static struct ButtonTable
{
  char *name;
  Boolean toggle;
  Widget w;
  void (*cb)();
} ButtonTable[] =
{
  { "quit",     False, NULL, Quit },
  { "open",	False, NULL, Open },
  { "home",	False, NULL, Home },
  { "back",	False, NULL, Back },
  { "reload",   False, NULL, Reload },
  { "cancel",   False, NULL, Cancel },
  { "dup",	False, NULL, Dup },
  { "help",	False, NULL, Help },
  { "addmark",	False, NULL, AddMark },
  { "viewmark", False, NULL, ViewMark },
  { "bookmark", False, NULL, Bookmark },
  { "search",   False, NULL, Search },
  { "source",   True,  NULL, Source },
  { "save",     False, NULL, Save },
  { NULL,       False, NULL, NULL},
};

extern AppResources appr;

static void
AddButtons(h, box, list)
CHead *h;
Widget box;
char *list;
{
  char name[256];
  struct ButtonTable *btp;

  while (sscanf(list, " %[^,]", name) == 1)
  {
    /*
     * Find the listed button and create its widget
     */
    for (btp = &ButtonTable[0]; btp->name != NULL; btp++)
    {
      if (!strcasecmp(btp->name, name))
      {
        if (btp->toggle)
        {
	  btp->w = XtVaCreateManagedWidget(btp->name,
					   toggleWidgetClass, box,
					   NULL);
        }
        else
        {
	  btp->w = XtVaCreateManagedWidget(btp->name,
					   commandWidgetClass, box,
					   NULL);
        }
	XtAddCallback(btp->w, XtNcallback, btp->cb, (XtPointer)h);
	break;
      }
    }

    /*
     * Fill in the entry in AppResources for the widget.
     */
    if (!strcasecmp(name, "open")) h->open = btp->w;
    else if (!strcasecmp(name, "back")) h->back = btp->w;
    else if (!strcasecmp(name, "reload")) h->reload = btp->w;
    else if (!strcasecmp(name, "cancel")) h->cancel = btp->w;
    else if (!strcasecmp(name, "quit")) h->quit = btp->w;
    else if (!strcasecmp(name, "home")) h->home = btp->w;
    else if (!strcasecmp(name, "help")) h->help = btp->w;
    else if (!strcasecmp(name, "dup")) h->dup = btp->w;
    else if (!strcasecmp(name, "addmark")) h->addmark = btp->w;
    else if (!strcasecmp(name, "viewmark")) h->viewmark = btp->w;
    else if (!strcasecmp(name, "source")) h->source = btp->w;
    else if (!strcasecmp(name, "save")) h->save = btp->w;
    else if (!strcasecmp(name, "search")) h->search = btp->w;
    else if (!strcasecmp(name, "bookmark")) h->bookmark = btp->w;

    /*
     * Skip to the next comma-delimited item in the list
     */
    while (*list && *list != ',')
      list++;
    if (*list == ',')
      list++;
  }

  return;
}

#define BUTTON_LIST "quit, open, home, back, reload, source, save, bookmark, dup, cancel"

/*
 * These are initialized in fallback.c
 */
#define offset(field) XtOffset(CHead *, field)
static XtResource       resource_list[] =
{
  { "button1Box", "BoxList", XtRString, sizeof(char *),
        offset(button1Box), XtRString, BUTTON_LIST },
  { "button2Box", "BoxList", XtRString, sizeof(char *),
        offset(button2Box), XtRString, NULL },
};

/*
 * CreateHead
 *
 * Setup chimera in a widget.
 */
CHead *
CreateHead(wc, name)
WWWContext wc;
char *name;
{
  Widget paned, box, form;
  CHead *h;
  Atom delete;
  MemPool mp;

  mp = MPCreate();
  h = (CHead *)MPCGet(mp, sizeof(CHead));
  h->mp = mp;
  h->wc = wc;

  GListAddHead(appr.heads, h);

  h->toplevel = XtVaAppCreateShell(name, "Chimera",
                                   topLevelShellWidgetClass,
                                   appr.dpy,
                                   NULL);

  XtGetApplicationResources(h->toplevel, h,
                            resource_list, XtNumber(resource_list),
                            NULL, 0);

  /*
   * Main window pane
   */
  paned = XtCreateManagedWidget("paned",
                                panedWidgetClass, h->toplevel, 
                                NULL, 0);

  /*
   * Button pane(s)
   */
  if (h->button1Box && *h->button1Box)
  {
    box = XtCreateManagedWidget("box1", boxWidgetClass, paned, NULL, 0);
    AddButtons(h, box, h->button1Box);
  }

  if (h->button2Box && *h->button2Box)
  {
    box = XtCreateManagedWidget("box2", boxWidgetClass, paned, NULL, 0);
    AddButtons(h, box, h->button2Box);
  }

  /*
   * URL pane
   */
  form = XtVaCreateManagedWidget("urlform", formWidgetClass, paned, NULL);
  XtVaCreateManagedWidget("urllabel", labelWidgetClass, form, NULL);
  h->url = XtVaCreateManagedWidget("url",
				   textfieldWidgetClass, form,
				   XtNstring, "",
				   NULL);
				   
  XtOverrideTranslations(h->url,
                         XtParseTranslationTable
                         ("<Key>Return: ReturnAction()"));

  /*
   * Message pane.
   */
  box = XtCreateManagedWidget("box4", boxWidgetClass, paned, NULL, 0);
  h->message = XtVaCreateManagedWidget("message",
                                       textfieldWidgetClass, box,
                                       NULL);

  /*
   * WWW pane.
   */
  h->wd = WWWCreateToplevelDisplay(wc, paned);

  h->www = (Widget)WWWDisplayToWidget(h->wd);

  h->urls = GListCreateX(h->mp);
  h->ourls = GListCreateX(h->mp);
  h->tmp = MPCreate();

  XtRealizeWidget(h->toplevel);

  delete = XInternAtom(XtDisplay(h->toplevel), "WM_DELETE_WINDOW", False);
  XSetWMProtocols (appr.dpy, XtWindow(h->toplevel), &delete, 1);
  XtOverrideTranslations (h->toplevel,
                          XtParseTranslationTable
                          ("<Message>WM_PROTOCOLS: DeleteAction()"));

  return(h);
}

/*
 * DestroyHead
 */
void
DestroyHead(h)
CHead *h;
{
  if (h->pending != NULL) WWWUnloadData(h->pending);
  if (h->wp != NULL) WWWUnloadData(h->wp);
  if (h->wn != NULL) WWWDestroyRender(h->wn);
  if (h->tmp != NULL) MPDestroy(h->tmp);
  GListRemoveItem(appr.heads, h);
  WWWDestroyDisplay(h->wd);
  XtDestroyWidget(h->toplevel);
  MPDestroy(h->mp);

  CheckForDone();

  return;
}

/*
 * ClearDialogValue
 */
static void
ClearDialogValue(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  if ((w = XtParent(w)) != NULL) MyDialogSetValue(w, "");

  return;
}

/*
 * CreateDialog
 */
Widget
CreateDialog(p, name, ofunc, dfunc, rfunc, closure)
Widget p;
char *name;
void (*ofunc)();
void (*dfunc)();
void (*rfunc)();
XtPointer closure;
{
  Widget w, dw;
  Window rw, cw;
  int rx, ry, wx, wy;
  unsigned int mask;

  XQueryPointer(appr.dpy, DefaultRootWindow(appr.dpy),
                &rw, &cw,
		&rx, &ry,
		&wx, &wy,
		&mask);
  w = XtVaCreatePopupShell(name,
			   transientShellWidgetClass, p,
			   XtNx, rx - 2,
			   XtNy, ry - 2,
			   NULL);
  dw = XtVaCreateManagedWidget("dialog",
			       mydialogWidgetClass, w,
			       XtNvalue, "",
			       NULL);
  MyDialogAddButton(dw, "ok", ofunc, closure);
  MyDialogAddButton(dw, "clear", ClearDialogValue, closure);
  MyDialogAddButton(dw, "dismiss", dfunc, closure);
  XtAddCallback(dw, XtNcallback, rfunc, closure);

  XtRealizeWidget(w);

  return(w);
}

/*
 * GetDialogWidget
 */
Widget
GetDialogWidget(w)
Widget w;
{
  return(XtNameToWidget(w, "dialog"));
}
