/*
 * plain.c
 *
 * libplain - a bad plain text renderer
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This sucks.
 */

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/Intrinsic.h>

#include "WWW.h"

#define PLAIN_MARGIN 20

typedef struct
{
  char *s;
  size_t len;
  int y;
  unsigned int width;
} LineInfoP, *LineInfo;

typedef struct
{
  MemPool mp;
  GList lines;
  size_t off;
  size_t len;
  byte *data;

  XFontStruct *font;
  Window win;
  Widget w;
  Display *dpy;
  GC gc;
  Pixel fg;

  WWWData wp;
  WWWContext wc;
  WWWDisplay wd;

  /* Dimensions stuff */
  unsigned int lineSpace;
  unsigned int baseline;
  unsigned int lineheight;
  unsigned int width, height;
} PlainInfoP, *PlainInfo;

static LineInfo
MakeLine(pi, s, len)
PlainInfo pi;
char *s;
size_t len;
{
  LineInfo li;

  li = (LineInfo)MPCGet(pi->mp, sizeof(LineInfoP));
  li->s = s;
  li->len = len;
  li->y = pi->height;
  pi->height += pi->lineheight;
  
  li->width = XTextWidth(pi->font, s, len);
  if (li->width > pi->width) pi->width = li->width + PLAIN_MARGIN * 2;

  WWWSetDimensions(pi->wd, pi->width, pi->height, true);

  XSetFont(pi->dpy, pi->gc, pi->font->fid);
  XSetForeground(pi->dpy, pi->gc, pi->fg);

  XDrawString(pi->dpy, pi->win, pi->gc, PLAIN_MARGIN, li->y + pi->baseline,
	      li->s,
	      (int)li->len); /* what to do about this ? */

  GListAddTail(pi->lines, li);

  return(li);
}

static void
ParsePlainData(pi)
PlainInfo pi;
{
  char *fcp, *cp, *lcp;
  GList list;

  WWWGetData(pi->wp, &(pi->data), &(pi->len), &list);

  fcp = (char *)(pi->data + pi->off);
  lcp = (char *)(pi->data + pi->len);
  for (cp = fcp; cp < lcp; cp++)
  {
    if (*cp == '\r')
    {
      MakeLine(pi, fcp, cp - fcp);
      pi->off = (byte *)cp - pi->data + 1;
      fcp = cp + 1;
      if (fcp < lcp && *fcp == '\n')
      {
	fcp++;
	pi->off++;
      }
    }
    else if (*cp == '\n')
    {
      MakeLine(pi, fcp, cp - fcp);
      pi->off = (byte *)cp - pi->data + 1;
      fcp = cp + 1;
    }
  }

  return;
}

static void
PlainAdd(closure)
void *closure;
{
  ParsePlainData((PlainInfo)closure);
  return;
}

static void
PlainEnd(closure)
void *closure;
{
  PlainInfo pi = (PlainInfo)closure;

  ParsePlainData(pi);

  if (pi->off < pi->len)
  {
    MakeLine(pi, (char *)(pi->data + pi->off), pi->len - pi->off);
  }

  pi->height += pi->lineheight;
  WWWSetDimensions(pi->wd, pi->width, pi->height, true);

  return;
}

static bool
PlainExpose(closure, x, y, width, height)
void *closure;
int x, y;
unsigned int width, height;
{
  LineInfo li;
  PlainInfo pi = (PlainInfo)closure;
 
  XSetFont(pi->dpy, pi->gc, pi->font->fid);
  XSetForeground(pi->dpy, pi->gc, pi->fg);

  for (li = (LineInfo)GListGetHead(pi->lines); li != NULL;
       li = (LineInfo)GListGetNext(pi->lines))
  {
    XDrawString(pi->dpy, pi->win, pi->gc, PLAIN_MARGIN, li->y + pi->baseline,
		li->s,
		(int)li->len ); /* what to do about this ? */
  }

  return(true);
}

static bool
PlainMouse(closure, x, y, button)
void *closure;
int x, y;
int button;
{
  return(true);
}

static bool
PlainMotion(closure, x, y, button)
void *closure;
int x, y;
int button;
{
  return(true);
}

/*
 * PlainQuery
 */
static byte *
PlainQuery(closure, key, size)
void *closure;
byte *key;
size_t *size;
{
  return(NULL);
}

/*
 * PlainSearch
 */
static int
PlainSearch(closure, data, len)
void *closure;
byte *data;
size_t len;
{
  return(0);
}

/*
 * PlainDestroy
 */
static void
PlainDestroy(closure)
void *closure;
{
  PlainInfo pi = (PlainInfo)closure;

  MPDestroy(pi->mp);

  return;
}

/*
 * PlainCancel
 */
static void
PlainCancel(closure)
void *closure;
{
  return;
}

/*
 * PlainInit
 */
static void *
PlainInit(wn, closure)
WWWRender wn;
void *closure;
{
  PlainInfo pi;
  MemPool mp;
  char *name;
  unsigned int width, height;
  WWWDisplayCallbacks dcb;
  WWWData wp;
  WWWDisplay wd;

  wd = WWWGetRenderDisplay(wn);
  wp = WWWGetRenderData(wn);

  mp = MPCreate();
  pi = (PlainInfo)MPCGet(mp, sizeof(PlainInfoP));
  pi->mp = mp;
  pi->wp = wp;
  pi->lineheight = 40;
  pi->lines = GListCreateX(mp);
  pi->wd = wd;
  pi->wc = WWWGetRenderContext(wn);

  pi->w = (Widget)WWWGetDrawWidget(wd, true);
  pi->dpy = XtDisplay(pi->w);
  pi->win = XtWindow(pi->w);
  pi->gc = DefaultGC(pi->dpy, DefaultScreen(pi->dpy));

  pi->lineSpace = 2;

  name = WWWGetResource(pi->wc, "plain.font");
  if (name == NULL) name = "fixed";
  if ((pi->font = XLoadQueryFont(pi->dpy, name)) == NULL)
  {
    fprintf (stderr, "Could not get default font.\n");
    fflush(stderr);
    return(NULL);
  }

  pi->baseline = pi->font->ascent + pi->lineSpace;
  pi->lineheight = pi->font->ascent + pi->font->descent + pi->lineSpace;

  pi->height = PLAIN_MARGIN;

  WWWGetNamedColor(wd, "black", &(pi->fg));

  XSetForeground(pi->dpy, pi->gc, pi->fg);
  
  WWWSetScrollBar(wd, true);
  WWWResetDisplay(wd);

  if (WWWGetDimensions(wd, &width, &height, true) == -1)
  {
    WWWSetDimensions(wd, width, height, false);
    /* Now grab the internal dimensions again */
    if (WWWGetDimensions(wd, &width, &height, true) == -1)
    {
      return(NULL);
    }
  }

  memset(&dcb, 0, sizeof(dcb));
  dcb.expose = PlainExpose;
  dcb.mouse = PlainMouse;
  dcb.motion = PlainMotion;
  WWWSetDisplayCallbacks(wd, &dcb, pi);

  return(pi);
}

/*
 * InitModule_Plain
 */
int
InitModule_Plain(wc)
WWWContext wc;
{
  WWWRenderHooks rh;

  memset(&rh, 0, sizeof(rh));
  rh.class_destroy = NULL;
  rh.content = "text/plain";
  rh.init = PlainInit;
  rh.add = PlainAdd;
  rh.end = PlainEnd;
  rh.destroy = PlainDestroy;
  rh.search = PlainSearch;
  rh.query = PlainQuery;
  rh.cancel = PlainCancel;
  WWWAddRenderHooks(wc, &rh);

  return(0);
}
