/*
 * WWW.h
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __WWW_H__
#define __WWW_H__ 1

#include "common.h"
#include "url.h"

/*
 * A whole bunch of types with which to cause trouble and confuse.
 */
typedef struct WWWSchemeP    *WWWScheme;
typedef struct WWWTaskP      *WWWTask;
typedef struct WWWStreamP    *WWWStream;
typedef struct WWWTimeOutP   *WWWTimeOut;
typedef struct WWWContextP   *WWWContext;
typedef struct WWWDisplayP   *WWWDisplay;
typedef struct WWWDataP      *WWWData;
typedef struct WWWAuthP      *WWWAuth;
typedef struct WWWAuthUIP    *WWWAuthUI;
typedef struct WWWRenderP    *WWWRender;
typedef unsigned long        WWWID;

typedef struct
{
  bool (*mouse) _ArgProto((void *, int, int, int));
  bool (*expose) _ArgProto((void *, int, int, unsigned int, unsigned int));
  bool (*redraw) _ArgProto((void *));
  bool (*resize) _ArgProto((void *));
  bool (*motion) _ArgProto((void *, int, int, int));
} WWWDisplayCallbacks;

typedef struct
{
  char *content;
  void *class_context;
  void *(*init) _ArgProto((WWWRender, void *));
  void (*add) _ArgProto((void *));
  void (*end) _ArgProto((void *));
  void (*destroy) _ArgProto((void *));
  void (*cancel) _ArgProto((void *));
  void (*class_destroy) _ArgProto((void *));
  byte *(*query) _ArgProto((void *, byte *, size_t *));
  int (*search) _ArgProto((void *, byte *, size_t));
} WWWRenderHooks;

typedef struct
{
  void *closure;
  void (*init) _ArgProto((WWWData, void *));
  void (*add) _ArgProto((void *));
  void (*end) _ArgProto((void *));
  void (*stop) _ArgProto((void *, char *));
  void (*message) _ArgProto((void *, char *));
} WWWSinkHooks;

typedef struct WWWRequestP WWWRequest;

typedef struct
{
  char *name;
  void *class_closure;
  void *(*init) _ArgProto((WWWScheme, WWWRequest *, void *));
  void (*stop) _ArgProto((void *));
  void (*destroy) _ArgProto((void *));
  void (*class_destroy) _ArgProto((void *));
  void (*getdata) _ArgProto((void *, byte **, size_t *, GList *));
  char *(*resolve_url) _ArgProto((MemPool, char *, char *));
} WWWSchemeHooks;

struct WWWRequestP
{
  MemPool mp;                /* */
  char *url;                 /* Absolute URL not-parsed */
  URLParts *up;              /* Absolute URL parsed */
  URLParts *pup;             /* Absolute proxy URL parsed */
  WWWSchemeHooks hooks;      /* hooks for module */
  void *input_data;          /* input data */
  size_t input_len;          /* input data len */
  char *input_type;          /* input data MIME content-type */
  char *input_method;        /* input method GET/POST */
  bool reload;               /* don't use cache? */
  GList contents;            /* acceptable contents, NULL = '*' */
};

typedef struct
{
  WWWRequest *wr;
  char *type;
} WWWRenderActionInfo;

typedef struct
{
  bool (*action) _ArgProto((void *, WWWRenderActionInfo *));
  bool (*resize) _ArgProto((void *));
  bool (*redraw) _ArgProto((void *));
  bool (*error) _ArgProto((void *, char *));
  bool (*message) _ArgProto((void *, char *));
} WWWRenderCallbacks;

/*
 * io.c
 */
typedef void (*WWWStreamCallback) _ArgProto((WWWStream, ssize_t, void *));
typedef void (*WWWTimeOutCallback) _ArgProto((WWWTimeOut, void *));

WWWStream WWWCreateINet _ArgProto((WWWContext, char *, int));
WWWStream WWWCreateINet2 _ArgProto((WWWContext));
WWWStream WWWCreateFDStream _ArgProto((WWWContext, int));
void WWWDestroyStream _ArgProto((WWWStream));
void WWWWriteStream _ArgProto((WWWStream, byte *, size_t,
			       WWWStreamCallback, void *));
void WWWReadStream _ArgProto((WWWStream, byte *, size_t,
			      WWWStreamCallback, void *));
int WWWGetINetPort _ArgProto((WWWStream));
unsigned long WWWGetINetAddr _ArgProto((WWWStream));

WWWTimeOut WWWCreateTimeOut _ArgProto((WWWContext, unsigned int,
				       WWWTimeOutCallback, void *));
void WWWDestroyTimeOut _ArgProto((WWWTimeOut));

/*
 * data.c
 */

/* Functions called by the scheme implementation */
void WWWInitScheme _ArgProto((WWWScheme, bool));
void WWWAddScheme _ArgProto((WWWScheme));
void WWWEndScheme _ArgProto((WWWScheme));
void WWWStopScheme _ArgProto((WWWScheme, char *));
void WWWSendSchemeMessage _ArgProto((WWWScheme, char *));
WWWContext WWWSchemeToContext _ArgProto((WWWScheme));

/* Functions called by the sink/scheme user */
WWWData WWWLoadData _ArgProto((WWWContext, WWWRequest *,
			       WWWSinkHooks *, void *));
void WWWUnloadData _ArgProto((WWWData));
void WWWCancelData _ArgProto((WWWData, bool));
void WWWGetData _ArgProto((WWWData, byte **, size_t *, GList *));
char *WWWGetDataInfo _ArgProto((WWWData, char *));
WWWContext WWWDataToContext _ArgProto((WWWData));
bool WWWDataWasReloaded _ArgProto((WWWData));

/*
 * request.c
 */
WWWRequest *WWWBuildRequest _ArgProto((WWWContext, char *, char *));
int WWWAddRegexContent _ArgProto((WWWContext, WWWRequest *, char *));
void WWWAddContent _ArgProto((WWWContext, WWWRequest *, char *));
bool WWWMatchContent _ArgProto((MemPool, char *, char *));
bool WWWMatchRequestContent _ArgProto((WWWRequest *, char *));
void WWWDestroyRequest _ArgProto((WWWRequest *));
void WWWReloadRequest _ArgProto((WWWRequest *, bool));
bool WWWCompareRequestURL _ArgProto((WWWRequest *, WWWRequest *));
bool WWWCompareRequestAccept _ArgProto((WWWRequest *, WWWRequest *));

/*
 * display.c
 */
void WWWSetDisplayCallbacks _ArgProto((WWWDisplay,
				      WWWDisplayCallbacks *, void *));
void WWWClearDisplayCallbacks _ArgProto((WWWDisplay));
void *WWWDisplayToClosure _ArgProto((WWWDisplay));
void *WWWDisplayToWidget _ArgProto((WWWDisplay));

int WWWGetPosition _ArgProto((WWWDisplay, int *, int *, bool));
int WWWGetDimensions _ArgProto((WWWDisplay,
				unsigned int *, unsigned int *, bool));
int WWWGetNamedColor _ArgProto((WWWDisplay, char *, unsigned long *));

WWWDisplay WWWCreateDisplay _ArgProto((WWWContext, WWWDisplay));

void WWWGetOnScreenDimensions _ArgProto((WWWDisplay,
					 int *, int *,
					 unsigned int *, unsigned int *));
int WWWSetPosition _ArgProto((WWWDisplay, int, int, bool));
void WWWSetDimensions _ArgProto((WWWDisplay,
				 unsigned int, unsigned int, bool));
int WWWDestroyDisplay _ArgProto((WWWDisplay));

void *WWWGetDrawWidget _ArgProto((WWWDisplay, bool));
void WWWSetScrollBar _ArgProto((WWWDisplay, bool));
void WWWResetDisplay _ArgProto((WWWDisplay));

WWWDisplay WWWCreateToplevelDisplay _ArgProto((WWWContext, void *));

void WWWExposeDisplay _ArgProto((WWWDisplay, int, int,
				 unsigned int, unsigned int));
void WWWSelectDisplay _ArgProto((WWWDisplay, int, int, int));
void WWWMotionDisplay _ArgProto((WWWDisplay, int, int, int));
void WWWRedrawDisplay _ArgProto((WWWDisplay));

/*
 * render.c
 */

/* Called by the render user/caller */
WWWRender WWWCreateRender _ArgProto((WWWContext,
				     WWWDisplay, WWWData, WWWRenderHooks *));
void WWWSetRenderCallbacks _ArgProto((WWWRender,
				      WWWRenderCallbacks *, void *));
void WWWAddRender _ArgProto((WWWRender));
void WWWEndRender _ArgProto((WWWRender));
void WWWDestroyRender _ArgProto((WWWRender));
void WWWCancelRender _ArgProto((WWWRender));
int WWWSearchRender _ArgProto((WWWRender, byte *, size_t));
byte *WWWQueryRender _ArgProto((WWWRender, byte *, size_t *));
WWWDisplay WWWGetRenderDisplay _ArgProto((WWWRender));
WWWData WWWGetRenderData _ArgProto((WWWRender));
WWWContext WWWGetRenderContext _ArgProto((WWWRender));

/* Called by the render implementation */
void WWWDoRenderAction _ArgProto((WWWRender, WWWRenderActionInfo *));
void WWWSendRenderMessage _ArgProto((WWWRender, char *));

/*
 * context.c
 */
typedef void (*WWWAuthCallback) _ArgProto((void *, char *, char *));

typedef void (*WWWAuthUICallback) _ArgProto((WWWContext, WWWAuthUI,
					     char *, char *,
					     void *));

WWWAuth WWWGetAuth _ArgProto((WWWContext, char *, char *,
			      WWWAuthCallback, void *));
void WWWReturnAuth _ArgProto((WWWAuthUI, char *, char *));
void WWWDestroyAuth _ArgProto((WWWAuth));
void WWWProvideAuthUI _ArgProto((WWWContext, WWWAuthUICallback, void *));

WWWContext WWWCreateContext _ArgProto((void *, char *, char *));
void WWWDestroyContext _ArgProto((WWWContext));

int WWWAddSchemeHooks _ArgProto((WWWContext, WWWSchemeHooks *));
int WWWAddRenderHooks _ArgProto((WWWContext, WWWRenderHooks *));

WWWContext WWWDisplayToContext _ArgProto((WWWDisplay));
WWWID WWWGetUniqueID _ArgProto((WWWContext));

WWWRenderHooks *WWWGetRenderHooks _ArgProto((WWWContext, char *));
WWWSchemeHooks *WWWGetSchemeHooks _ArgProto((WWWContext, char *));

typedef void (*WWWTaskProc) _ArgProto((void *));

WWWTask WWWScheduleTask _ArgProto((WWWContext, WWWTaskProc, void *));
void WWWRemoveTask _ArgProto((WWWContext, WWWTask));

char *WWWExt2Content _ArgProto((WWWContext, char *));
void WWWReadTypeFiles _ArgProto((WWWContext, char *));

/*
 * resource.c
 */
char *WWWGetResource _ArgProto((WWWContext, char *));
int WWWAddResourceFile _ArgProto((WWWContext, char *));
int WWWAddResourceString _ArgProto((WWWContext, char *));
char *WWWGetBoolResource _ArgProto((WWWContext, char *, bool *));
char *WWWGetIntResource _ArgProto((WWWContext, char *, int *));
char *WWWGetUIntResource _ArgProto((WWWContext, char *, unsigned int *));
char *WWWGetFileResource _ArgProto((WWWContext, MemPool, char *));

#endif
