/*
 * WWWP.h
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __WWWP_H__
#define __WWWP_H__

#include "WWW.h"

/*
 * Xt widget type definitions
 */
typedef struct _WWWClassRec  *WWWWidgetClass;
typedef struct _WWWRec	     *WWWWidget;

extern WidgetClass           wwwWidgetClass;

typedef struct WWWCacheP     *WWWCache;

/*
 * WWW definitions
 */
struct WWWContextP
{
  int              maxiocnt;          /* maximum IOs */
  bool             printLoadMessages;
  MemPool          mp;                /* memory pool */
  XrmDatabase      db;                /* message database */

  WWWCache         cc;

  /* Lists of stuff we need to know about */
  GList            sourcehooks;
  GList            renderhooks;
  GList            mimelist;
  GList            renderlist;
  GList            sources;
  GList            tolist;
  GList            translist;

  /* Task stuff */
  GList            tasks;
  GList            oldtasks;
  XtWorkProcId     wpid;

  /* */
  WWWAuthUICallback authui_callback;
  void             *authui_closure;

  XtAppContext     appcon;            /* X application context */
  WWWID            id;                /* next unique ID */
};

/*
 * More Xt definitions
 */
typedef struct _WWWClassPart
{
  XtPointer extension;
} WWWClassPart;

typedef struct _WWWClassRec
{
  CoreClassPart	        core_class;
  CompositeClassPart	composite_class;
  WWWClassPart	        www_class;
} WWWClassRec;

extern WWWClassRec wwwClassRec;

typedef struct _WWWPart
{
  /* Public */
  Pixel       foreground;
  Pixel       background;

  /* X Private */
  Widget      vert_bar, horiz_bar;  /* scroll bars */
  Widget      clip;                 /* clips the child */
  Widget      child;                /* the drawing window */
  Widget      menu;                 /* menu widget */

  /* other private */
  WWWDisplay  wd;
} WWWPart;

typedef struct _WWWRec
{
  CorePart		core;
  CompositePart     	composite;
  WWWPart		www;
} WWWRec;

/*
 *
 * Prototypes
 *
 */

/*
 * WWW.c
 */
Widget WWWDrawWidget _ArgProto((Widget));
int WWWSetSize _ArgProto((Widget, bool, unsigned int, unsigned int));
int WWWSetDrawSize _ArgProto((Widget, bool, unsigned int, unsigned int));
int WWWGetSize _ArgProto((Widget, bool, unsigned int *, unsigned int *));
void WWWMoveChild _ArgProto((WWWWidget, int, int));

/*
 * action.c
 */
void WWWExposeAction _ArgProto((Widget, XEvent *, String *, Cardinal *));
void WWWSelectAction _ArgProto((Widget, XEvent *, String *, Cardinal *));
void WWWMotionAction _ArgProto((Widget, XEvent *, String *, Cardinal *));

/*
 * cache.c
 */
WWWCache WWWCacheInit _ArgProto((WWWContext));
void WWWCacheDestroy _ArgProto((WWWCache));
bool WWWIsDiskCached _ArgProto((WWWCache, char *));
int WWWWriteCache _ArgProto((WWWCache, WWWScheme));
WWWSchemeHooks *WWWGetCacheHooks _ArgProto((WWWCache));

/*
 * data.c
 */
void WWWGetSchemeData _ArgProto((WWWScheme, byte **, size_t *, GList *));

#endif
