/*
 * context.c
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "WWWP.h"

struct WWWTaskP
{
  WWWTaskProc     func;
  void            *closure;
  bool            removed;
};

struct WWWAuthUIP
{
  MemPool         mp;
  WWWAuth         wa;
};

struct WWWAuthP
{
  MemPool         mp;
  WWWAuthCallback callback;
  void            *closure;
  WWWAuthUI       wu;
};

static Boolean ScheduleTaskProc _ArgProto((XtPointer));

WWWContext
WWWCreateContext(crap, fallbacks, dbfiles)
void *crap;
char *fallbacks;
char *dbfiles;
{
  WWWContext wc;
  MemPool mp;
  char *f, *filename;

  mp = MPCreate();
  wc = (WWWContext)MPCGet(mp, sizeof(struct WWWContextP));
  wc->mp = mp;
  wc->appcon = (XtAppContext)crap;
  wc->sources = GListCreateX(mp);
  wc->sourcehooks = GListCreateX(mp);
  wc->renderhooks = GListCreateX(mp);
  wc->mimelist = GListCreateX(mp);
  wc->tasks = GListCreateX(mp);
  wc->oldtasks = GListCreateX(mp);
  wc->tolist = GListCreateX(mp);

  WWWAddResourceString(wc, fallbacks);

  f = dbfiles;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    WWWAddResourceFile(wc, filename);
  }

/*
  wc->translist = GListCreate();
  if ((tfiles = WWWGetResource(wc, "www.translationFiles")) != NULL)
  {
    f = tfiles;
    while ((filename = mystrtok(f, ':', &f)) != NULL)
    {
      URLReadTranslations(wc->translist, filename);
    }
  }
*/
  
  wc->cc = WWWCacheInit(wc);

  if (WWWGetIntResource(wc, "www.maxDownloads", &wc->maxiocnt) == NULL)
  {
    wc->maxiocnt = 4;
  }
  else if (wc->maxiocnt <= 1) wc->maxiocnt = 1;

  if (WWWGetBoolResource(wc, "www.printLoadMessages",
			 &wc->printLoadMessages) == NULL)
  {
    wc->printLoadMessages = false;
  }

  return(wc);
}

WWWID
WWWGetUniqueID(wc)
WWWContext wc;
{
  return(wc->id++);
}

void
WWWDestroyContext(wc)
WWWContext wc;
{
  WWWRenderHooks *rhooks;
  WWWSchemeHooks *shooks;
  GList list;

  if (wc->cc != NULL) WWWCacheDestroy(wc->cc);

  list = wc->renderhooks;
  for (rhooks = (WWWRenderHooks *)GListGetHead(list); rhooks != NULL;
       rhooks = (WWWRenderHooks *)GListGetNext(list))
  {
    if (rhooks->class_destroy != NULL)
    {
      (rhooks->class_destroy)(rhooks->class_context);
    }
  }
  list = wc->sourcehooks;
  for (shooks = (WWWSchemeHooks *)GListGetHead(list); shooks != NULL;
       shooks = (WWWSchemeHooks *)GListGetNext(list))
  {
    if (shooks->class_destroy != NULL)
    {
      (shooks->class_destroy)(shooks->class_closure);
    }
  }
  if (wc->db != NULL) XrmDestroyDatabase(wc->db);

  MPDestroy(wc->mp);

  return;
}

/*
 * WWWAddSchemeHooks
 */
int
WWWAddSchemeHooks(wc, shooks)
WWWContext wc;
WWWSchemeHooks *shooks;
{
  WWWSchemeHooks *nhooks;

  myassert(shooks->name != NULL, "Protocol not set in data hooks.");

  myassert(shooks->init != NULL, "Init function is NULL");
  myassert(shooks->stop != NULL, "Stop function is NULL");
  myassert(shooks->destroy != NULL, "Destroy function is NULL");
  myassert(shooks->getdata != NULL, "Getdata function is NULL");

  nhooks = (WWWSchemeHooks *)MPCGet(wc->mp, sizeof(WWWSchemeHooks));
  memcpy(nhooks, shooks, sizeof(WWWSchemeHooks));
  
  GListAddHead(wc->sourcehooks, nhooks);

  return(0);
}

/*
 * WWWAddRenderHooks
 */
int
WWWAddRenderHooks(wc, rhooks)
WWWContext wc;
WWWRenderHooks *rhooks;
{
  WWWRenderHooks *nr;

  nr = (WWWRenderHooks *)MPCGet(wc->mp, sizeof(WWWRenderHooks));
  memcpy(nr, rhooks, sizeof(WWWRenderHooks));

  GListAddHead(wc->renderhooks, nr);

  return(0);
}

/*
 * WWWGetAuth
 */
WWWAuth
WWWGetAuth(wc, message, username, callback, closure)
WWWContext wc;
char *message;
char *username;
WWWAuthCallback callback;
void *closure;
{
  WWWAuth wa;
  WWWAuthUI wu;
  MemPool mp;

  if (wc->authui_callback == NULL) return(NULL);

  mp = MPCreate();
  wa = (WWWAuth)MPCGet(mp, sizeof(struct WWWAuthP));
  wa->mp = mp;
  wa->callback = callback;
  wa->closure = closure;

  mp = MPCreate();
  wu = (WWWAuthUI)MPCGet(mp, sizeof(struct WWWAuthUIP));
  wu->mp = mp;
  wu->wa = wa;

  wa->wu = wu;

  if (wc->authui_callback != NULL)
  {
    (wc->authui_callback)(wc, wu, message, username, wc->authui_closure);
  }

  return(wa);
}

/*
 * WWWReturnAuth
 */
void
WWWReturnAuth(wu, username, password)
WWWAuthUI wu;
char *username;
char *password;
{
  if (wu->wa != NULL)
  {
    (wu->wa->callback)(wu->wa->closure, username, password);
  }
  MPDestroy(wu->mp);
  return;
}

/*
 * WWWProvideAuthUI
 */
void
WWWProvideAuthUI(wc, callback, closure)
WWWContext wc;
WWWAuthUICallback callback;
void *closure;
{
  wc->authui_callback = callback;
  wc->authui_closure = closure;
  return;
}

/*
 * WWWDestroyAuth
 */
void
WWWDestroyAuth(wa)
WWWAuth wa;
{
  if (wa->wu != NULL) wa->wu->wa = NULL;
  MPDestroy(wa->mp);
  return;
}

/*
 * WWWGetRenderHooks
 */
WWWRenderHooks *
WWWGetRenderHooks(wc, content)
WWWContext wc;
char *content;
{
  WWWRenderHooks *rh;
  size_t rlen;
  GList list;

  if (content == NULL) return(NULL);

  rlen = strlen(content);

  list = wc->renderhooks;
  for (rh = (WWWRenderHooks *)GListGetHead(list); rh != NULL;
       rh = (WWWRenderHooks *)GListGetNext(list))
  {
    if (strlen(rh->content) == rlen && strcasecmp(rh->content, content) == 0)
    {
      return(rh);
    }
  }

  return(NULL);
}

/*
 * WWWGetSchemeHooks
 */
WWWSchemeHooks *
WWWGetSchemeHooks(wc, proto)
WWWContext wc;
char *proto;
{
  WWWSchemeHooks *shooks;
  size_t plen;
  GList list;

  plen = strlen(proto);

  list = wc->sourcehooks;
  for (shooks = (WWWSchemeHooks *)GListGetHead(list); shooks != NULL;
       shooks = (WWWSchemeHooks *)GListGetNext(list))
  {
    if (strlen(shooks->name) == plen &&
	strcasecmp(shooks->name, proto) == 0)
    {
      return(shooks);
    }
  }

  return(NULL);
}

/*
 * ScheduleTaskProc
 */
static Boolean
ScheduleTaskProc(closure)
XtPointer closure;
{
  WWWContext wc = (WWWContext)closure;
  WWWTask wt;

  while ((wt = (WWWTask)GListPop(wc->tasks)) != NULL)
  {
    if (!wt->removed) (wt->func)(wt->closure);
    GListAddTail(wc->oldtasks, wt);
  }

  wc->wpid = 0;

  return(True);
}

/*
 * WWWScheduleTask
 */
WWWTask
WWWScheduleTask(wc, func, closure)
WWWContext wc;
WWWTaskProc func;
void *closure;
{
  WWWTask wt;

  if ((wt = (WWWTask)GListPop(wc->oldtasks)) == NULL)
  {
    wt = (WWWTask)MPCGet(wc->mp, sizeof(struct WWWTaskP));
  }
  wt->func = func;
  wt->closure = closure;
  wt->removed = false;

  GListAddTail(wc->tasks, wt);

  if (wc->wpid == 0)
  {
    wc->wpid = XtAppAddWorkProc(wc->appcon, ScheduleTaskProc, wc);
    myassert(wc->wpid != 0, "Made bad assumption about XtAppAddWorkProc");
  }

  return(wt);
}

/*
 * WWWRemoveTask
 */
void
WWWRemoveTask(wc, wt)
WWWContext wc;
WWWTask wt;
{
  GList list;
  WWWTask c;

  list = wc->tasks;
  for (c = (WWWTask)GListGetHead(list); c != NULL;
       c = (WWWTask)GListGetNext(list))
  {
    if (c == wt)
    {
      c->removed = true;
      break;
    }
  }

  return;
}

struct WWWType
{
  char *content;
  char *ext;
};

static struct WWWType def_mtlist[] =
{
  { "text/html", "html" },
  { "text/html", "htm" },
  { "text/plain", "txt" },
  { "image/gif", "gif" },
  { "image/png", "png" },
  { "image/xbm", "xbm" },
  { "image/x-xpixmap", "xpm" },
  { "image/x-portable-anymap", "pnm" },
  { "image/x-portable-bitmap", "pbm" },
  { "image/x-portable-graymap", "pgm" },
  { "image/x-portable-pixmap", "ppm" },
  { "image/jpeg", "jpg" },
  { "image/jpeg", "jpeg" },
  { "image/tiff", "tiff" },
  { "image/tiff", "tif" },
  { "image/x-fits", "fit" },
  { "image/x-fits", "fits" },
  { "image/x-fits", "fts" },
  { "image/x-png", "png" },
  { "image/png", "png" },
  { "audio/basic", "au" },
  { "audio/basic", "snd" },
  { "text/x-compress-html", "html.Z" },
  { "text/x-gzip-html", "html.gz" },
  { "application/postscript", "ps" },
  { "application/x-dvi", "dvi" },
  { "application/x-gzip", "gz" },
  { "application/x-compress", "Z" },
  { "application/x-tar", "tar" },
  { "video/mpeg", "mpeg" },
  { "video/mpeg", "mpg" },
  { NULL, NULL },
};

/*
 * WWWReadTypes
 */
void
WWWReadTypeFiles(wc, filelist)
WWWContext wc;
char *filelist;
{
  FILE *fp;
  char *f;
  char *cp;
  char buffer[256];
  char content[256];
  char exts[256];
  struct WWWType *c;
  char *filename, *e;
  MemPool mp;

  mp = MPCreate();

  wc->mimelist = GListCreate();

  f = filelist;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    filename = FixPath(mp, filename);
    if (filename == NULL) continue;

    fp = fopen(filename, "r");
    if (fp == NULL) continue;
    
    while (fgets(buffer, sizeof(buffer), fp))
    {
      if (buffer[0] == '#' || buffer[0] == '\n') continue;
      
      if (sscanf(buffer, "%s %[^\n]", content, exts) == 2)
      {
	cp = exts;
	while ((e = mystrtok(cp, ' ', &cp)) != NULL)
	{
	  c = (struct WWWType *)alloc_mem(sizeof(struct WWWType));
	  memset(c, 0, sizeof(struct WWWType));
	  c->content = alloc_string(content);
	  c->ext = alloc_mem(strlen(e) + 2);
	  strcpy(c->ext, ".");
	  strcat(c->ext, e);

	  GListAddTail(wc->mimelist, c);
	}
      }
    }
    
    fclose(fp);
  }

  MPDestroy(mp);

  return;
}

/*
 * WWWExt2Content
 */
char *
WWWExt2Content(wc, ext)
WWWContext wc;
char *ext;
{
  int elen, flen;
  struct WWWType *c;
  int i;
  GList list;

  list = wc->mimelist;
  flen = strlen(ext);
  for (c = (struct WWWType *)GListGetHead(list); c != NULL;
       c = (struct WWWType *)GListGetNext(list))
  {
    elen = strlen(c->ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, c->ext, elen) == 0)
      {
	return(c->content);
      }
    }
  }

  for (i = 0; def_mtlist[i].content != NULL; i++)
  {
    elen = strlen(def_mtlist[i].ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, def_mtlist[i].ext, elen) == 0)
      {
	return(def_mtlist[i].content);
      }
    }
  }

  return(NULL);
}
