/*
 * resource.c
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "WWWP.h"

int
WWWAddResourceString(wc, str)
WWWContext wc;
char *str;
{
  XrmDatabase db;

  db = XrmGetStringDatabase(str);
  if (wc->db != NULL) XrmMergeDatabases(db, &(wc->db));
  else wc->db = db;

  if (wc->db == NULL) return(-1);

  return(0);
}

int
WWWAddResourceFile(wc, file)
WWWContext wc;
char *file;
{
  char *dbfile;
  MemPool mp;
  XrmDatabase db;

  if (file == NULL) return(-1);

  mp = MPCreate();
  if ((dbfile = FixPath(mp, file)) == NULL)
  {
    MPDestroy(mp);
    return(-1);
  }

  db = XrmGetFileDatabase(dbfile);
  if (wc->db != NULL) XrmMergeDatabases(db, &(wc->db));
  else wc->db = db;

  MPDestroy(mp);

  if (wc->db == NULL) return(-1);

  return(0);
}

/*
 * WWWGetResource
 */
char *
WWWGetResource(wc, name)
WWWContext wc;
char *name;
{
  char *type;
  XrmValue v;

  if (wc->db == NULL) return(NULL);
  if (!XrmGetResource(wc->db, name, XtNstring, &type, &v)) return(NULL);
  return(v.addr);
}

/*
 * WWWGetFileResource
 */
char *
WWWGetFileResource(wc, mp, name)
WWWContext wc;
MemPool mp;
char *name;
{
  char *value;

  if ((value = WWWGetResource(wc, name)) == NULL) return(NULL);
  return(FixPath(mp, value));
}

/*
 * WWWGetBoolResource
 */
char *
WWWGetBoolResource(wc, name, value)
WWWContext wc;
char *name;
bool *value;
{
  char *str;

  if ((str = WWWGetResource(wc, name)) == NULL) return(NULL);

  *value = false;
  if (strncasecmp("true", str, 4) == 0) *value = true;
  if (atoi(str) > 0) *value = true;

  return(str);
}

/*
 * WWWGetIntResource
 */
char *
WWWGetIntResource(wc, name, value)
WWWContext wc;
char *name;
int *value;
{
  char *str;

  if ((str = WWWGetResource(wc, name)) == NULL) return(NULL);

  *value = atoi(str);

  return(str);
}

/*
 * WWWGetUIntResource
 */
char *
WWWGetUIntResource(wc, name, value)
WWWContext wc;
char *name;
unsigned int *value;
{
  char *str;
  int bla;

  if ((str = WWWGetResource(wc, name)) == NULL) return(NULL);

  bla = atoi(str);
  if (bla < 0) bla = 0;
  *value = bla;

  return(str);
}
