/*
 * timeout.c
 *
 * Copyright (C) 1994-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>

#include <X11/IntrinsicP.h>

#include "WWWP.h"

struct WWWTimeOutP
{
  MemPool mp;
  WWWContext wc;
  XtIntervalId iid;
  WWWTimeOutCallback func;
  void *closure;
};

static void TimeOutHandler _ArgProto((XtPointer, XtIntervalId *));

/*
 * WWWCreateTimeOut
 */  
WWWTimeOut
WWWCreateTimeOut(wc, interval, func, closure)
WWWContext wc;
unsigned int interval;
WWWTimeOutCallback func;
void *closure;
{
  WWWTimeOut pto;
  MemPool mp;

  mp = MPCreate();
  pto = (WWWTimeOut)MPCGet(mp, sizeof(struct WWWTimeOutP));
  pto->mp = mp;
  pto->wc = wc;
  pto->closure = closure;

  GListAddHead(wc->tolist, pto);

  pto->func = func;
  pto->iid = XtAppAddTimeOut(pto->wc->appcon,
			     (unsigned long)interval,
			     TimeOutHandler, pto->wc);
  
  return(pto);
}

/*
 * TimeOutHandler
 */
static void
TimeOutHandler(closure, iid)
XtPointer closure;
XtIntervalId *iid;
{
  WWWContext wc = (WWWContext)closure;
  WWWTimeOut pto;

  for (pto = (WWWTimeOut)GListGetHead(wc->tolist); pto != NULL;
       pto = (WWWTimeOut)GListGetNext(wc->tolist))
  {
    if (pto->iid == *iid)
    {
      pto->iid = 0;
      (pto->func)(pto, pto->closure);
      return;
    }
  }

  myassert(false, "Timeout not found."); /* wha? */

  return;
}

/*
 * WWWDestroyTimeOut
 */
void
WWWDestroyTimeOut(pto)
WWWTimeOut pto;
{
  GListRemoveItem(pto->wc->tolist, pto);
  if (pto->iid != 0) XtRemoveTimeOut(pto->iid);
  MPDestroy(pto->mp);
  return;
}
